import { ListenerRss } from "listener-rss";

import connect, { DatabaseConnection, sql } from "@databases/sqlite";

export class SqliteTools {
  path?: string;

  constructor(path?: string) {
    this.path = path;
  }

  async withDB<Type>(
    callback: (db: DatabaseConnection) => Promise<Type>
  ): Promise<Type> {
    const db = connect(this.path);
    try {
      return callback(db);
    } finally {
      await db.dispose();
    }
  }

  async ensureTableExists() {
    await this.withDB(async (db) => {
      let req = sql`CREATE TABLE IF NOT EXISTS listeners
                    (
                      address TEXT NOT NULL,
                      customfields TEXT DEFAULT '[]' NOT NULL,
                      timeloop INTEGER DEFAULT 300 NOT NULL,
                      last_entries_links TEXT DEFAULT '[]' NOT NULL,
                      PRIMARY KEY (address),
                      CHECK(timeloop >= 0)
                    );`;

      await db.query(req);
    });
  }

  async fetchAll(): Promise<ListenerRss.Config[]> {
    const rows = await this.withDB(async (db) => {
      let req = sql`SELECT *
                    FROM listeners`;
      return await db.query(req);
    });

    return rows.map((row: any) => ({
      address: row["address"],
      customfields: JSON.parse(row["customfields"]),
      timeloop: row["timeloop"],
      lastEntriesLinks: JSON.parse(row["last_entries_links"]),
    }));
  }

  async insertListener(listener: ListenerRss.Config) {
    await this.withDB(async (db) => {
      let req = sql`INSERT INTO listeners (address, timeloop, customfields, last_entries_links)
                    VALUES (${listener.address},
                      ${listener.timeloop},
                      ${JSON.stringify(listener.customfields ?? [])},
                      ${JSON.stringify(listener.lastEntriesLinks ?? [])})`;
      await db.query(req);
    });
  }

  async deleteListener(adr: string) {
    await this.withDB(async (db) => {
      let req = sql`DELETE FROM listeners
                    WHERE address = ${adr}`;
      await db.query(req);
    });
  }

  async updateAll(listeners: ListenerRss.Config[]) {
    await this.withDB(async (db) => {
      await db.tx(async (transaction) => {
        for (const listener of listeners) {
          let req = sql`UPDATE listeners
                      SET last_entries_links = ${JSON.stringify(
                        listener.lastEntriesLinks
                      )}
                      WHERE address = ${listener.address}`;
          await transaction.query(req);
        }
      });
    });
  }
}
